# ett_parser.py

from typing import List

import requests
import sys
import aiohttp
import asyncio


def __url(path, legacy_api=False) -> str:
    return (
               "https://www.elevenvr.club/" if legacy_api else "https://elevenvr.club/api/v1/"
           ) + path


def __exception_handler(func):
    def inner_function(*args, **kwargs):
        try:
            response = func(*args, **kwargs)
            response.raise_for_status()
            # Code here will only run if the request is successful
            return response
        except requests.exceptions.HTTPError as errh:
            print(errh)
        except requests.exceptions.ConnectionError as errc:
            print(errc)
        except requests.exceptions.Timeout as errt:
            print(errt)
        except requests.exceptions.RequestException as err:
            print(err)

    return inner_function


def get_user(user_id) -> List:
    @__exception_handler
    def request_user(user_id):
        return requests.get(__url(f"accounts/{user_id}/", legacy_api=True))

    return request_user(user_id).json()["data"]["attributes"]


def get_friends(user_id) -> List:
    @__exception_handler
    def request_friends(user_id):
        return requests.get(__url(f"accounts/{user_id}/", legacy_api=True) + "friends")

    return request_friends(user_id).json()["data"]


def get_matches_legacy(user_id, unranked=False) -> List:
    @__exception_handler
    def request_matches(user_id, unranked):
        unranked_str = "true" if unranked else "false"
        return requests.get(
            __url(f"accounts/{user_id}/matches/?unranked={unranked_str}")
        )

    return request_matches(user_id, unranked).json()["data"]


def get_matches(user_id, unranked=False) -> List:
    async def request_matches(session, url):
        async with session.get(url) as resp:
            matches_page = await resp.json()
            return matches_page["data"]

    async def get_matches_async(user_id, unranked):
        unranked_str = "true" if unranked else "false"
        async with aiohttp.ClientSession() as session:
            tasks = []
            res = requests.get(__url(f"accounts/{user_id}/matches/?unranked={unranked_str}")).json()
            last_page = res["links"]["last"]
            num_pages = int(last_page.split('page%5Bnumber%5D=')[1].split('&')[0])
            if num_pages == 1:
                matches = [res["data"]]
            else:
                for number in range(1, num_pages + 1):
                    url = (
                        __url(f"accounts/{user_id}/matches/?page%5Bnumber%5D={number}&unranked={unranked_str}")
                    )
                    tasks.append(asyncio.ensure_future(request_matches(session, url)))

                matches = await asyncio.gather(*tasks)
            return matches

    res = asyncio.run(get_matches_async(user_id, unranked))
    all_pages = res[0]
    for page in res[1:]:
        all_pages.extend(page)

    return all_pages


def get_elo_history(user_id) -> List:
    @__exception_handler
    def request_elo_history(user_id):
        return requests.get(
            __url(f"accounts/{user_id}/", legacy_api=True) + "elo-history"
        )

    return request_elo_history(user_id).json()["data"]


def get_leaderboard(num_players=10):
    async def request_leaderboard(session, url):
        async with session.get(url) as resp:
            lb_page = await resp.json()
            return lb_page["scores"]

    async def get_leaderboard_async(num_players):
        async with aiohttp.ClientSession() as session:
            tasks = []
            for number in range(0, num_players // 10 + 1):
                url = (
                        __url(f"leaderboards/", legacy_api=True)
                        + "?start="
                        + str(number * 10)
                )
                print(url)
                tasks.append(asyncio.ensure_future(request_leaderboard(session, url)))

            lb = await asyncio.gather(*tasks)
            return lb

    res = asyncio.run(get_leaderboard_async(num_players))
    all_pages = res[0]
    for page in res[1:]:
        all_pages.extend(page)

    return all_pages[:num_players]


def user_search(username) -> List:
    async def request_user(session, url):
        async with session.get(url) as resp:
            user_page = await resp.json()
            return user_page["data"]

    async def get_user_async(username):
        async with aiohttp.ClientSession() as session:
            tasks = []
            res = requests.get(__url(f"accounts/search/{username}/", legacy_api=True)).json()
            last_page = res["links"]["last"]
            num_pages = int(last_page.split('page%5Bnumber%5D=')[1].split('&')[0])
            if num_pages == 1:
                users = [res["data"]]
            else:
                for number in range(1, num_pages + 1):
                    url = (
                        __url(f"accounts/search/{username}?page%5Bnumber%5D={number}", legacy_api=True)
                    )
                    tasks.append(asyncio.ensure_future(request_user(session, url)))

                users = await asyncio.gather(*tasks)
            return users

    res = asyncio.run(get_user_async(username))
    all_pages = res[0]
    for page in res[1:]:
        all_pages.extend(page)

    return all_pages



# pyETT.py

from typing import List

import pandas as pd
from pyETT import ett_parser


class Player:
    """
    A class to represent a Player.
    """

    HOME = 0
    AWAY = 1

    def __get_user(self):
        return ett_parser.get_user(self.id)

    def __init__(self, user_id, player=None, legacy_api=False):
        self.id = user_id
        if player is None:
            player = self.__get_user()
            legacy_api = True
        self.name = player["user-name"] if legacy_api else player["username"]
        self.elo = player["elo"]
        self.rank = player["rank"]
        self.wins = player["wins"]
        self.losses = player["losses"]
        self.last_online = player["last-online"]
        self.friends = self.matches = self.elo_history = None

    def __str__(self):
        return self.name

    def get_friends(self) -> List["Player"]:
        """
        Return a player’s friends list
        """
        if self.friends is None:
            res = ett_parser.get_friends(self.id)
            if not res:
                self.friends = None
            else:
                self.friends = [
                    Player(user_id=v["id"], player=v["attributes"], legacy_api=True)
                    for v in res
                ]
        return self.friends

    def get_friends_dataframe(self) -> pd.DataFrame:
        """
        Return a player’s friends list in a dataframe
        """
        return pd.DataFrame([vars(u) for u in self.get_friends()]).dropna(
            how="all", axis="columns"
        )

    def get_matches(self, unranked=False) -> List["Match"]:
        """
        Return player’s matches.
        Currently, limited to the 25 most recent matches.
        """
        if self.matches is None:
            res = ett_parser.get_matches(self.id, unranked)
            if not res:
                self.matches = None
            else:
                matches = [Match(match_id=v["id"], match=v["attributes"]) for v in res]
        return matches

    def get_matches_dataframe(self, unranked=False) -> pd.DataFrame:
        """
        Return player’s matches in a pandas dataframe.
        Currently, limited to the 25 most recent matches.
        """
        return pd.DataFrame([vars(m) for m in self.get_matches(unranked)])

    def get_elo_history(self) -> pd.DataFrame:
        """
        Returns player’s elo score history.
        """
        if self.elo_history is None:
            res = ett_parser.get_elo_history(self.id)
            if not res:
                self.elo_history = None
            else:
                dt, elo = map(
                    list,
                    zip(
                        *[
                            (
                                v["attributes"]["created-at"],
                                v["attributes"]["current-elo"],
                            )
                            for v in res
                        ]
                    ),
                )

                self.elo_history = pd.DataFrame({"elo": elo}, index=dt)

        return self.elo_history


class Match:
    """
    A class to represent a Match.
    """

    class Round:
        """
        A class to represent a round of a Match.
        """

        def __init__(self, round_attributes):
            self.id = round_attributes["id"]
            self.round_number = round_attributes["round-number"]
            self.state = round_attributes["state"]
            self.away_score = round_attributes["away-score"]
            self.home_score = round_attributes["home-score"]
            self.winner = round_attributes["winner"]
            self.created_at = round_attributes["created-at"]

    def __init__(self, match_id, match):
        self.created_at = match["created-at"]
        self.id = match_id

        self.ranked = match["ranked"]
        self.number_of_rounds = match["number-of-rounds"]
        self.state = match["state"]
        self.winning_team = match["winning-team"]
        self.losing_team = match["losing-team"]
        self.home_score = match["home-score"]
        self.away_score = match["away-score"]

        home_player_index = 0 if match["players"][0]["team"] == Player.HOME else 1
        self.home_player = Player(
            match["players"][home_player_index]["id"],
            match["players"][home_player_index],
        )
        self.away_player = Player(
            match["players"][1 - home_player_index]["id"],
            match["players"][1 - home_player_index],
        )

        self.rounds = [self.Round(r) for r in match["rounds"]]

    def get_rounds_dataframe(rounds: List["round"]) -> pd.DataFrame:
        """
        Converts a list of rounds to a DataFrame
        """
        return pd.DataFrame([vars(r) for r in rounds])


class ETT:
    """
    A class to represent Eleven Table Tennis (ETT).
    """

    def __init__(self):
        self.leaderboard = None

    def user_search(self, username, perfect_match=False) -> List[Player]:
        """
        Returns a list of players whose name contains username, if perfect_match is False.
        Otherwise, it returns a list of players whose usernames is a perfect match with username.
        """
        res = ett_parser.user_search(username)

        if not res:
            users = None
        else:
            users = [
                Player(user_id=v["id"], player=v["attributes"], legacy_api=True)
                for v in res
                if (
                    not perfect_match
                    or (perfect_match and v["attributes"]["user-name"] == username)
                )
            ]

        return users

    def user_search_dataframe(self, username, perfect_match=False) -> pd.DataFrame:
        """
        Returns a list of players whose name contains username, if perfect_match is False.
        Otherwise, it returns a list of players whose usernames is a perfect match with username.
        """
        return pd.DataFrame(
            [vars(u) for u in self.user_search(username, perfect_match)]
        ).dropna(how="all", axis="columns")

    def get_leaderboard(self, num_players=10) -> List[Player]:
        """
        Returns a list of players from the leaderboard. Currently, limited to Top 10 players.
        """
        if self.leaderboard is None:
            res = ett_parser.get_leaderboard(num_players)
            if not res:
                self.leaderboard = None
            else:
                self.leaderboard = [
                    self.user_search(v["username"], perfect_match=True)[0] for v in res
                ]
        return self.leaderboard

    def get_leaderboard_dataframe(self, num_players=10) -> pd.DataFrame:
        """
        Returns a pandas dataframe with players from the leaderboard. Currently, limited to Top 10 players.
        """
        lb = pd.DataFrame([vars(u) for u in self.get_leaderboard(num_players)]).dropna(
            how="all", axis="columns"
        )
        # Overwriting rank as currently user API rank is lagged compared to the
        # rank in leaderboard API
        lb["rank"] = lb.index
        return lb



# -*- coding: utf-8 -*-

# Learn more: https://pyett.readthedocs.io/

from setuptools import setup, find_packages


with open("README.md") as fh:
    readme = fh.read()

with open("LICENSE.md") as f:
    license = f.read()

setup(
    name="pyETT",
    version="0.1.6",
    description="Python library for Eleven VR Table Tennis data",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Tharsis T. P. Souza",
    url="https://pyett.readthedocs.io/",
    license=license,
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    install_requires=["requests==2.22.0", "pandas==1.0.3", "nbsphinx==0.8.6"],
)
